/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#if !defined(_ASYNC_SOCKET_HPP_INCLUDED_)
#define _ASYNC_SOCKET_HPP_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ip_addr.hpp"

namespace network
{
	class async_socket
	{
	public:
		async_socket(unsigned long _timeout = 0/*infinite*/);
		virtual ~async_socket();

		bool create(
			int nSocketType = SOCK_STREAM,
			long lEvent = FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE,
			const ip_addr& addr =  ip_addr(0,0,0,0,0));
		bool select(long lEvent = FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE );
		void close();
		int get_last_error() const;
		bool shutdown(int how = SD_BOTH);
		bool listen(int backlog = SOMAXCONN);
		bool attach(SOCKET s, const network::ip_addr& _peername = network::ip_addr());
		bool b_auto_delete;
		bool read(char* pBuff, DWORD len);
		bool write(char* pBuff, DWORD len);

		void set_priority(int pr = THREAD_PRIORITY_NORMAL );
	protected:
		virtual void on_close(int err);
		virtual void on_accept(int err);
		virtual void on_read(int err);
		virtual void on_write(int err);

		virtual void on_attach();

		virtual void on_read_complete(DWORD err, DWORD len);
		virtual void on_write_complete(DWORD err, DWORD len);
		
		virtual void on_clear();

		virtual void on_timeout();

		bool bind(const ip_addr& p_addr);

		struct accepted_socket
		{
			SOCKET s;
			network::ip_addr addr;
		};

		accepted_socket accept() const;

		network::ip_addr ip;
		network::ip_addr peername;
	private:
		unsigned long timeout;
		process::thread socket_thread;
		SOCKET the_socket;
		WSAEVENT hEvent;
		WSAEVENT hStopEvent;
		OVERLAPPED readO;
		OVERLAPPED writeO;
		long l_events;
		static int static_thread_func(void*);
		int socket_func();

		bool init();
		void _on_close(int err);
		void _on_connect(int err);
		void _on_read(int err);
		void _on_write(int err);
		void _on_accept(int err);
		void _on_read_complete();
		void _on_write_complete();
		void _on_clear();
		void _on_timeout();
	};
}

#endif //_ASYNC_SOCKET_HPP_INCLUDED_
