/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#include "http_server.hpp"
#include "http_date.hpp"
#include "request_socket.h"
#include "sinchro.hpp"
#include "thread.hpp"
#include "listen_socket.h"


#define WM_FREE_REQUEST WM_USER+1
#define WM_ACCEPT_REQUEST WM_USER+2
#define WM_STOP WM_USER+3

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

listen_socket::listen_socket(
		const std::string& _ip/* = "0.0.0.0:80"*/,
		const std::string& _root_dir/* = "."*/,
		const std::string& _index_file_name/* = std::string()*/,
		bool _allow_directory_browsing/* = false*/,
		size_t max_conn/* = 100*/,
		unsigned long timeout/* = 60*/):
max_concurent_connections(max_conn),
connection_timeout(timeout),
root_dir(_root_dir),
index_file_name(_index_file_name),
allow_directory_browsing(_allow_directory_browsing),
hMessageQueueCreatedEvent(NULL)
{
	ip=_ip;
	hMessageQueueCreatedEvent=CreateEvent(NULL,FALSE,FALSE,NULL);
}

listen_socket::~listen_socket()
{
	stop();

	CloseHandle(hMessageQueueCreatedEvent);
}

void 
listen_socket::set_max_conn(size_t max_conn/* = 100*/)
{
	process::auto_lock lock(object_cs);
	max_concurent_connections=max_conn;
}

void 
listen_socket::set_conn_timeout(unsigned long timeout/* = 60*/)
{
	process::auto_lock lock(object_cs);
	connection_timeout=timeout;
}

bool 
listen_socket::start(UINT port, char* addr,  const std::string& _host_name)
{
	process::auto_lock lock(object_cs);

	::ResetEvent(hMessageQueueCreatedEvent);

	free_tread.create(static_free_thread_func, this);

	::WaitForSingleObject(hMessageQueueCreatedEvent, INFINITE);
	
	if(_host_name.size()==0){

		char szHostName[1024];

		if(gethostname(szHostName, sizeof(szHostName))==0){

			struct hostent* p_host_ent=gethostbyname(
				szHostName);
			int err = WSAGetLastError();

			host_name="http://";

			if(p_host_ent!=0){
				host_name+=p_host_ent->h_name;
			}else{
				host_name+=szHostName;
			}
		}
	}else{
		host_name=_host_name;
	}

	if(create(SOCK_STREAM, FD_ACCEPT|FD_CLOSE, ip)==true){
		if(listen() == true){
			srv_out
				<<"Binded to "
				<<std::string(ip)
				<<std::endl
				<<"Host name: "
				<<host_name
				<<std::endl;

			set_priority(+2);
			return true;
		}
	}

	return false;
}

void 
listen_socket::on_accept(int err)
{
	process::auto_lock lock(object_cs);

	size_t conn_count=request_socket::get_conn_count();

	if(conn_count>=max_concurent_connections){
	}

	accepted_socket s=accept();

	if(s.s!=INVALID_SOCKET){
		waitong_requests.push(s);
		free_tread.post_message(WM_ACCEPT_REQUEST, 0);
	}
}

void 
listen_socket::set_index_file_name(const std::string& index)
{
	index_file_name=index;
}

void 
listen_socket::stop()
{
	close();

	free_tread.post_message(WM_STOP);

	request_socket::wait_for_all_close();

	free_tread.post_message(WM_QUIT);

	free_tread.wait_for_end();

	while(waitong_requests.size()>0){
		::closesocket(waitong_requests.front().s);
		waitong_requests.pop();
	}
}

void 
listen_socket::static_request_call_back_fn(request_socket::call_back_reason reason, const request_socket* r_socket, const void* p)
{
	reinterpret_cast<listen_socket*>
		(const_cast<void*>(p))->
		request_call_back_fn(reason, r_socket);
}

void 
listen_socket::request_call_back_fn(request_socket::call_back_reason reason, const request_socket* p)
{
	if(reason == request_socket::destroyed){
		{
			process::auto_lock lock(object_cs);
			busy_set.erase(p);
		}

		free_tread.post_message(WM_FREE_REQUEST, (WPARAM)p);
	}
}

int 
listen_socket::static_free_thread_func(void* p)
{
	return reinterpret_cast<listen_socket*>(p)->free_thread_func();
}

int 
listen_socket::free_thread_func()
{
	MSG msg;

	free_tread.set_priority(+1);

	::PeekMessage(&msg, NULL, WM_USER, WM_USER, PM_NOREMOVE);
	
	::SetEvent(hMessageQueueCreatedEvent);
	
	while(::GetMessage(&msg, NULL, 0, 0)){
		if(msg.message == WM_FREE_REQUEST){
			delete ((request_socket*)msg.wParam);
		}else if(msg.message == WM_ACCEPT_REQUEST){

			accepted_socket sock;

			{
				process::auto_lock lock(object_cs);
				sock=waitong_requests.front();
				waitong_requests.pop();
			}

			request_socket* p=new request_socket(
				host_name,
				root_dir, 
				index_file_name,
				allow_directory_browsing,
				connection_timeout,
				static_request_call_back_fn,
				this);

			{
				process::auto_lock lock(object_cs);
				busy_set.insert(p);
			}

			p->attach(sock.s, sock.addr);

		}else if(msg.message == WM_STOP){
			process::auto_lock lock(object_cs);
			for(busy_request_iterator_t it=busy_set.begin();
			it!=busy_set.end();
			it++){
				const_cast<request_socket*>(*it)->close();
			}
		}
	}
	
	return 0;
}
