/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#include "http_server.hpp"
#include "request_socket.h"
#include "listen_socket.h"
#include "prog_opt.hpp"
#include <signal.h>

using namespace std;

static option option_table[]=
{
	option("root-dir", "./", "root directory"),
	option("ip-addr", "0.0.0.0:80", "server ip address & port number"),
	option("index-file", "index.html", "default document(s)"),
	option("allow-dir", "yes", "allow directory listing"),
	option("time-out", "0", "connection timeout (s), 0 - infinite")
};

static void print_copyright(){

	srv_out
		<<szAppName<<" "
		<<szCopyright
		<<endl;
}

static HANDLE hStopEvent=NULL;

static void __cdecl signal_processor( int sig)
{
	if(sig==SIGINT){
		srv_out<<"CTRL-C signal occured."<<endl;
	}else{
		srv_out<<"Unexpected signal occured."<<endl;
	}
	srv_out
		<<"Shuting down the server..."
		<<endl;

	SetEvent(hStopEvent);
}

int main(int argc, char* argv[]){

	SetConsoleOutputCP(GetACP());

	hStopEvent=CreateEvent(NULL,TRUE,FALSE,NULL);

	if(hStopEvent==NULL)
		return -1;
		
	server_globals_init();
	prog_options options(
		option_table, 
		sizeof(option_table)/sizeof(option));

	print_copyright();
	if(argc<2){
		srv_out
			<<"Usage: http_server.exe [[-option value]...]"
			<<endl
			<<"options:"
			<<endl;
		options.print_usage();
	}else{
		options.parse_cmd_line(argc, argv);
	}

	{
		WSADATA data;
		if(WSAStartup(MAKEWORD( 2, 2 ), &data) != 0){
			srv_err
				<<"Can't initialise WinSock 2.0"<<endl;
			return -1;
		}
	}

	{
		std::string root=option_table[0].get_value();
		
		if(root==option_table[0].get_default()){
			char szTmp[0x1000];
			if(GetCurrentDirectory(sizeof(szTmp),szTmp) != 0){
				int len=strlen(szTmp);
				for(int i=0; i<len; i++){
					if(szTmp[i]=='\\') szTmp[i]='/';
				}
				root=szTmp;
			}
		}
		
		if(root.size()>0){
			char c=root[root.size()-1];
			if(c=='/' || c=='\\'){
				root.erase(root.size()-1);
			}
		}
		
		option_table[0].set_value(root);
	}

	srv_out<<endl<<"Current settings:"<<endl;
	options.print_values();
	srv_out<<endl;

	signal(SIGINT, signal_processor);

	{
		listen_socket listen_sock(
			option_table[1].get_value(),
			option_table[0].get_value(),
			option_table[2].get_value(),
			option_table[3].get_value().compare("yes") == 0,
			100,
			atol(option_table[4].get_value().c_str())*1000);
		
		if(listen_sock.start() == false){
			srv_err
				<<"Can't start server! "
				<<"error code: "
				<<listen_sock.get_last_error()
				<<endl;
		}else{
			
			string	str_buf;
			
			srv_out
				<<"Waiting for incoming connections..."
				<<endl
				<<"For shuting down the server press CTRL-C"
				<<endl<<endl;
			
			WaitForSingleObject(hStopEvent, INFINITE);
			
			listen_sock.close();
		}
	}
	
	CloseHandle(hStopEvent);

	WSACleanup();

	return 0;
}
