/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#include "http_server.hpp"
#include "http_date.hpp"
#include "request.h"
#include <sstream>


request::request():
type(unknown),
http_version_minor(0), 
http_version_major(0),
beg_pos(0),
end_pos(0),
instance_length(0),
Entity_Too_Large(false),
Request_URI_Too_Long(false),
keep_alive(false),
request_processed(0)
{

}

request::~request()
{

}

bool 
request::get_line(char* *p_buff, 
		int* len, char* *p_line, int* line_size)
{
	char *p_beg=*p_buff;
	char *p_end=*p_buff;
	int chars=0;

	*p_line=0;
	*line_size=0;

	if(*len<=0)
		return false;

	while(*len>0 && *p_beg!=0 && (*p_beg=='\r' || *p_beg=='\n' || *p_beg==' ' || *p_beg=='\t')){
		p_beg++;
		(*len)--;
	}

	p_end=p_beg;
	
	while((*len)>0 && *p_end!=0 && *p_end!='\r' && *p_end!='\n'){
		p_end++;
		(*len)--;
		chars++;
	}

	*p_buff=p_end;
	if(*len>0){
		(*p_buff)++;
		(*len)--;
	}

	if(chars>0){
		*p_end=0;
		*p_line=p_beg;
		*line_size=chars;
		return true;
	}else{
		return false;
	}
}

bool 
request::interpret_line(char* line, int line_size)
{
	std::istringstream strbuff(line, line_size);
	std::string str;
	std::string str_for_cmp;

	while(strbuff.eof() == false){
		strbuff>>str;
		str_for_cmp.erase();
		str_for_cmp.append(str);
		_strlwr(const_cast<char*>(str_for_cmp.c_str()));

		if(type==unknown){
			if(str_for_cmp == "get"){
				type=get;
				strbuff>>requested_target;

				if(requested_target.size()>512)
					Request_URI_Too_Long=true;

				strbuff>>str;
			
				_strlwr(const_cast<char*>(str.c_str()));

				if(str.compare(0,5,"http/") == 0){
					str.erase(0,5);
					int pos_dote=str.find('.');
					str_for_cmp.erase();
					str_for_cmp.append(str, 0, pos_dote);
					http_version_major=atoi(str_for_cmp.c_str());
					str.erase(0,pos_dote+1);
					http_version_minor=atoi(str.c_str());
				}
			}else if(str_for_cmp == "head"){
				type=head;
				strbuff>>requested_target;

				if(requested_target.size()>512)
					Request_URI_Too_Long=true;

				strbuff>>str;
			
				_strlwr(const_cast<char*>(str.c_str()));

				if(str.compare(0,5,"http/") == 0){
					str.erase(0,5);
					int pos_dote=str.find('.');
					str_for_cmp.erase();
					str_for_cmp.append(str, 0, pos_dote);
					http_version_major=atoi(str_for_cmp.c_str());
					str.erase(0,pos_dote+1);
					http_version_minor=atoi(str.c_str());
				}
			}
		}else{
			if(str_for_cmp == "range:" || str_for_cmp == "content-range:"){

				beg_pos=0;
				end_pos=0;
				instance_length=0;

				strbuff>>str_for_cmp;
				_strlwr(const_cast<char*>(str_for_cmp.c_str()));

				str_for_cmp.erase(0, str_for_cmp.find('=', 0)+1);
				{
					std::string start_pos;
					int minus_pos=str_for_cmp.find('-', 0);
					if(minus_pos>0){
						start_pos.append(str_for_cmp, 0, minus_pos);
						str_for_cmp.erase(0, minus_pos+1);
						beg_pos=atol(start_pos.c_str());
					}
				}
				if(str_for_cmp.size()>0){
					std::string str_end_pos;
					int back_splash_pos=str_for_cmp.find('/', 0);
					if(back_splash_pos>0){
						str_end_pos.append(str_for_cmp, 0, back_splash_pos);
						str_for_cmp.erase(0, back_splash_pos+1);
						end_pos=atol(str_end_pos.c_str());
						end_pos++;
					}
				}
				if(str_for_cmp.size()>0){
					instance_length=atol(str_for_cmp.c_str());
				}
			}else if(str_for_cmp == "host:"){
				host="http://";
				strbuff>>str_for_cmp;
				host+=str_for_cmp;
			}else if(str_for_cmp == "if-modified-since:"){
				
				std::string date;
				for(int i=0; i<6; i++){
					if(strbuff.eof() == false){
						strbuff>>str_for_cmp;
						date+=str_for_cmp;
						date+=' ';
					}else
						break;
				}

				remote_file_date = http_date(date);
				if(type==get){
					type=get_if_modified;
				}
			}else if(str_for_cmp == "connection:"){
				strbuff>>str_for_cmp;
				_strlwr(const_cast<char*>(str_for_cmp.c_str()));
				if(str_for_cmp == "close"){
					keep_alive=false;
				}else if(str_for_cmp == "keep-alive"){
					keep_alive=true;
				}
			}
		}
	}

	return true;
}

bool 
request::interpret_request(const char* buff, size_t len)
{
	char* p_buff=const_cast<char*>(buff);
	int length=len;
	int line_size;
	char* p_line;

	if(len>0x1000-1){
		Entity_Too_Large=true;
		return true;
	}

	while(get_line(&p_buff, &length, &p_line, &line_size) == true){
		interpret_line(p_line, line_size);
	}

	request_processed++;
	
	return true;
}

void 
request::clear_request()
{
	type=unknown;
	beg_pos=0;
	end_pos=0;
	instance_length=0;
	Entity_Too_Large=false;
	Request_URI_Too_Long=false;
	keep_alive=false;
}

bool 
request::request_timeout()
{
	clear_request();
	type=timeout;
	return true;
}
