/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#include "http_server.hpp"
#include "http_date.hpp"
#include "request_socket.h"
#include <string>
#include <set>


static HANDLE hNotHaveConnections=NULL;

class instanse_init
{
public:
	instanse_init(){
		hNotHaveConnections=CreateEvent(NULL, TRUE, TRUE, NULL);
	}

	~instanse_init(){
		CloseHandle(hNotHaveConnections);
	}
};

static instanse_init tnstanse;

static unsigned long request_count=0;
process::critical_section request_socket::class_cs;

void request_socket::wait_for_all_close()
{
	WaitForSingleObject(hNotHaveConnections,INFINITE);
}

unsigned long 
request_socket::get_conn_count()
{
	process::auto_lock lock(class_cs);
	return request_count;
	
}

void 
request_socket::add_ref(request_socket* request)
{
	process::auto_lock lock(class_cs);
	request_count++;
	ResetEvent(hNotHaveConnections);
}

void 
request_socket::release(request_socket* p_This)
{
	process::auto_lock lock(class_cs);
	request_count--;
	if(request_count==0){
		SetEvent(hNotHaveConnections);
	}

}

request_socket::request_socket(
		const std::string& _host_name,
		const std::string& _root_dir/* = "."*/,
		const std::string _index_file_name/* = std::string()*/,
		bool _allow_directory_browsing/* = true*/,
		unsigned long _timeout/* = 60*/,
		const call_back_fn _call_back/* = 0*/,
		const void* _call_back_param/* = 0*/):
async_socket(_timeout),
request(),
cb_sended(0),
cb_readed(0),
call_back(_call_back),
call_back_param(_call_back_param),
respond(_host_name, _root_dir, _index_file_name, _allow_directory_browsing)
{
//	b_auto_delete=true;
	add_ref(this);
}

request_socket::~request_socket()
{
	release(this);
}

void 
request_socket::on_attach()
{
//	select(FD_CLOSE|FD_READ);
	log_str=std::string(peername);
	log_str+=' ';

	SYSTEMTIME time;
	GetSystemTime(&time);

	std::string str_time=std::string(http_date(time));
	
	log_str+=str_time;

	if(read(readBuff, sizeof(readBuff))==false)
		close();	
}

void 
request_socket::on_close(int err)
{
	async_socket::close();
}

void 
request_socket::on_read(int err)
{
}

void 
request_socket::on_write(int err)
{
}

void 
request_socket::on_read_complete(DWORD err, DWORD len)
{
	if(err==0 && len>0){
		if(cb_readed!=0){
			cb_readed=0;
		}
		
		cb_readed+=len;
		
		if(interpret_request(readBuff, len) == true){
			if(make_respond(*this)==true){
				on_write_complete(0, 0x1000);
				return;
			}
		}
		
	}

	close();
}

void 
request_socket::on_write_complete(DWORD err, DWORD writen)
{
	if(err == 0){
		char* buff;
		int len;
		
		if(get_buffer_for_send(&buff, &len) == true){
			
			if(write(buff, len) == false){
				DWORD err = GetLastError();
				if(err == ERROR_NETNAME_DELETED){
					close();
					return ;
				}else{
					goto Leave;
				}
			}else{
				cb_sended+=len;
				move_pointers(writen);
				return;
			}
		}else{
			if(keep_alive==false){
				goto Leave;
			}else{
				clear();
				clear_request();
				if(read(readBuff, sizeof(readBuff))==true)
					return;	
			}
		}
	}

Leave:
	
	close();
}

void 
request_socket::on_clear()
{
	write_to_cout_log(log_str);

	shutdown();
	clear();
	if(call_back!=0)
		call_back(destroyed, this, call_back_param);
}

void 
request_socket::on_timeout()
{
	if(request_timeout()==true){
		if(make_respond(*this)==true){
			on_write_complete(0, 0x1000);
			return;
		}
	}

	close();
}
