/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#include "http_server.hpp"
#include "http_date.hpp"
#include "request.h"
#include "respond.h"
#include <malloc.h>

using namespace std;

static const char szHttp_endl[]="\r\n";
static const char szHttp_1_0[]="HTTP/1.0";
static const char szHttp_1_1[]="HTTP/1.1";

static const char sz200[]="OK";
static const char sz206[]="Partial content";

static const char sz302[]="Object Moved";
static const char sz304[]="Not Modified";

static const char sz403[]="Acces Forbidden";
static const char sz404[]="Not Found";
static const char sz405[]="Method Not Allowed"; 
static const char sz408[]="Request Timeout"; 
static const char sz414[]="Request-URI Too Long";
static const char sz416[]="Requested Range Not Satisfiable";

static const char sz501[]="Not Implemented";
static const char szHTML_BR[]="<br>";

static const char* ContentTypes[]={
	"application/octet-stream",
	"text/html",
	"application/pdf",
	"image/gif",
	"application/msword",
	"application/x-rar-compressed",
	"text/plain",
	"image/jpeg",
	"application/zip",
	"application/x-mswrite"
};

typedef struct
{
	char *ext;
	cont_types_t type;
}file_ext_to_cont_type_t;

static const file_ext_to_cont_type_t FileExtensions[]={
	"htm", html,
	"html", html,
	"gif", gif,
	"rar", rar,
	"zip", zip,
	"exe", unknown,
	"tgz", unknown,
	"pdf", pdf,
	"doc", msword,
	"txt", txt,
	"h", txt,
	"hpp", txt,
	"c", txt,
	"cpp", txt,
	"cxx", txt,
	"pas", txt,
	"asm", txt,
	"s", txt,
	"for", txt,
	"jpeg", jpeg,
	"jpg", jpeg,
	"wri", wri
};

static const bool is_html_simbol[256]=
{
//		0	1	2	3	4	5	6	7	8	9	a	b	c	d	e	f
/*0*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*1*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*2*/	0,	1,	1,	0,	1,	0,	0,	1,	1,	1,	1,	0,	1,	1,	1,	1,
/*3*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*4*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*5*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*6*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*7*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*8*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*9*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*a*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*b*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*c*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*d*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*e*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,
/*f*/	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1,	1
};

static 
const std::string
to_http_string(const std::string& str)
{
	string temp;

	int len=str.size();
	unsigned char c;
	char szTmp[10];
	for(int i=0; i<len; i++){
		c=str[i];
		if(is_html_simbol[c] == false){
			wsprintf(szTmp, "%%%02X", c);
			temp+=szTmp;
		}else{
			temp+=str[i];
		}
	}

	return temp;
}

static 
const std::string
from_http_string(const std::string& http_str)
{
	string temp;

	int len=http_str.size();
	unsigned char c;
	char szHex[3];
	
	for(int i=0; i<len; i++){
		c=http_str[i];
		if( c=='%' ){
			char* hexEndP;
			if(i<=len-3){
				i++;
				szHex[0]=http_str[i++];
				szHex[1]=http_str[i];
				szHex[2]=0;
				unsigned long cr=strtoul(szHex, &hexEndP, 16);
				if(cr<256){
					c=cr;
				}
			}
		}
		temp+=c;
	}

	return temp;
}

respond::respond(
		const std::string _host_name,
		const std::string _root_dir,
		const std::string _index_file_name,
		bool _allow_directory_browsing/* = true*/):
hFile(INVALID_HANDLE_VALUE),
err_code(0),
file_size(0),
file_pos(0),
file_end_pos(0),
start_from(0),
cb_buffered(0),
buff_pos(0),
content_type(unknown),
host_name(_host_name),
root_dir(_root_dir),
allow_directory_browsing(_allow_directory_browsing),
index_file_name(_index_file_name)
{
	
}

respond::~respond()
{
	if(hFile != INVALID_HANDLE_VALUE){
		CloseHandle(hFile);
	}
}

bool 
respond::make_respond(request& rqu)
{
	char szTmp[1024];
	char szHttpErr[128];

	b_have_body=false;

	if(rqu.http_version_major == 1 && 
		rqu.http_version_minor == 1){
		respond_str=szHttp_1_1;
	}else{
		respond_str=szHttp_1_0;
	}

	if(rqu.host.size()>0){
		host_name=rqu.host;
	}

	{
		SYSTEMTIME sys_time;
	
		GetSystemTime(&sys_time);
		str_date=std::string(http_date(sys_time));
	}

	if(rqu.type==request::get || 
		rqu.type==request::get_if_modified ||
		rqu.type==request::head){
		
		if(rqu.type==request::head)
			log_str+="\nHEAD ";
		else
			log_str+="\nGET ";

		bool b_dir(false);
		log_str+=from_http_string(rqu.requested_target);
		if(rqu.Request_URI_Too_Long==true){
			uri_too_long(rqu, szHttpErr);
		}else
		if(parse_url(rqu.requested_target, &b_dir)==false){
			if(not_found_stuff(rqu, szHttpErr) == false)
				return false;
		}else{
			
			what_content_type(rqu);
			
			if(b_dir == true && index_file_name.size()==0 && allow_directory_browsing == true){
				if(ditectory_stuff(rqu, szHttpErr) == false){
					if(not_found_stuff(rqu, szHttpErr) == false)
						return false;
				}
			}else{
				if(open_file(rqu, szHttpErr, b_dir) == false){
					if(b_dir == true && index_file_name.size()>0){
						if(allow_directory_browsing){
							if(ditectory_stuff(rqu, szHttpErr) == false){
								if(not_found_stuff(rqu, szHttpErr) == false)
									return false;
							}
						}else{
							if(forbidden_stuff(rqu, szHttpErr, b_dir)==false)
								return false;
						}
					}else{
						if(b_dir && redirect_stuff(rqu, szHttpErr)==false){
							if(not_found_stuff(rqu, szHttpErr) == false)
								return false;
						}else{
							if(not_found_stuff(rqu, szHttpErr) == false)
								return false;
						}
					}
				}
			}
		}
	}else if(rqu.type==request::timeout){
		log_str+="\nRequest timeout ";
		if(request_timeout_stuff(rqu, szHttpErr)==false)
			return false;
	}else{
		log_str+="\nNot implemented ";
		if(not_implemented_stuff(rqu, szHttpErr)==false)
			return false;
	}

	wsprintf(szTmp, " %u", err_code);
	log_str+=szTmp;

	respond_str+=szTmp;
	respond_str+=szHttpErr;
	respond_str+=szHttp_endl;		
	
	respond_str+=add_to_header;
		
	respond_str+="Server: ";
	respond_str+=szAppName;
	respond_str+=' ';
	respond_str+=szCopyright;
	respond_str+=szHttp_endl;

	respond_str+="Date: ";
	respond_str+=str_date;
	respond_str+=szHttp_endl;

	respond_str+="Connection: ";
	if(rqu.keep_alive == true){
		respond_str+="Keep-Alive";
	}else{
		respond_str+="close";
	}
	respond_str+=szHttp_endl;

	if(b_have_body==true){

		if(hFile != INVALID_HANDLE_VALUE){
			std::string temp;
			content_type_to_str(content_type, &temp);
			respond_str+="Content-Type: ";
			respond_str+=temp;
			respond_str+=szHttp_endl;
		}
		
		if(rqu.http_version_major == 1 && 
			rqu.http_version_minor == 1 && 
			(
			err_code == 200 ||
			err_code == 206
			)){
			respond_str+="Accept-Ranges: bytes";
			respond_str+=szHttp_endl;
		}
		
		if(hFile != INVALID_HANDLE_VALUE){
			respond_str+="Last-Modified: ";
			respond_str+=std::string(http_date(file_time));
			respond_str+=szHttp_endl;
		}
		
		
		respond_str+="Content-length: ";
		if(hFile != INVALID_HANDLE_VALUE){
			wsprintf(szTmp, "%u", file_end_pos-start_from);
		}
		respond_str+=szTmp;
		respond_str+=szHttp_endl;
		
		if(hFile != INVALID_HANDLE_VALUE){
			if(rqu.http_version_major == 1 && 
				rqu.http_version_minor == 1 && err_code == 206){
				respond_str+="Content-Range: bytes ";
				wsprintf(szTmp, "%u-%u/%u", 
					start_from,
					file_end_pos-1,
					file_size);
				respond_str+=szTmp;
				respond_str+=szHttp_endl;
			}
		}
		
	}

	respond_str+=szHttp_endl;
	err_code=0;

	strcpy(buffer, respond_str.c_str());
	cb_buffered=strlen(buffer);

	if(cb_buffered == 0){
		move_pointers(0);
	}

	return true;
}

void 
respond::add_header_to_stuff(
	const std::string& head,
	const std::string& title)
{
	stuff_str="<html><head>";
	stuff_str+="<title>";

	stuff_str+=head;

	stuff_str+="</title>";
	stuff_str+="</head><body>";
	stuff_str+="<h2>";

	stuff_str+=title;

	stuff_str+="</h2><hr><pre>";
}

void 
respond::close_stuff()
{
	//stuff_str+="</blockquote><hr width=\"100%\">";

	stuff_str+="</pre><hr>";
	stuff_str+="<a href=\"http://rukhov.newmail.ru\">";
	stuff_str+=szAppName;
	stuff_str+="</a>";
	stuff_str+=". ";
	stuff_str+=szCopyright;
	stuff_str+=szHTML_BR;
	stuff_str+=szHttp_endl;
	stuff_str+="BUG reporting: <a href=\"mailto: r_ukhov@yahoo.com\">r_ukhov@yahoo.com</a><br>";
	stuff_str+="Generated ";
	stuff_str+=str_date;
	stuff_str+="</body></html>";
	stuff_str+=szHttp_endl;
}

bool 
respond::get_buffer_for_send(char* *pp_buff, int* p_len)
{
	if(cb_buffered>0){
		*pp_buff=&buffer[buff_pos];
		*p_len=cb_buffered;
		return true;
	}

	return false;
}

static unsigned g_readed=0;

void 
respond::move_pointers(int how_mach)
{
	int to_read;

	if(hFile==INVALID_HANDLE_VALUE){
		cb_buffered=0;
		buff_pos=0;
		return ;
	}

	if(file_pos<file_end_pos){
		to_read=RESPOND_BUFF_SIZE;

		if(to_read>(file_end_pos-file_pos))
			to_read=(file_end_pos-file_pos);

		DWORD readed;

		if(ReadFile(
			hFile,
			buffer,
			to_read,
			&readed,
			NULL)==TRUE){
			cb_buffered=readed;
			g_readed+=readed;
			file_pos+=readed;

			if(file_pos == file_end_pos){
				CloseHandle(hFile);
				hFile=INVALID_HANDLE_VALUE;
			}
			return;
		}
	}
		
	cb_buffered=0;
	buff_pos=0;

	CloseHandle(hFile);
	hFile=INVALID_HANDLE_VALUE;
	file_pos=file_end_pos;
}

static char* DeysOfWeek[]={
	"Sun",
	"Mon",
	"Tue",
	"Wed",
	"Thu",
	"Fri",
	"Sat"
};

static char* Months[]={
	"",
	"Jan",
	"Feb",
	"Mar",
	"Apr",
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Okt",
	"Nov",
	"Dec"
};

void 
respond::make_date_time_str(const SYSTEMTIME* time, std::string* str)
{
	char szTmp[128];

	wsprintf(szTmp, "%s, %u %s %u %02u:%02u:%02u GMT",
		DeysOfWeek[time->wDayOfWeek],
		time->wDay,
		Months[time->wMonth],
		time->wYear,
		time->wHour,
		time->wMinute,
		time->wSecond);

	*str=szTmp;
}

bool 
respond::get_file_time(HANDLE h_File, SYSTEMTIME* time)
{
	FILETIME f_time;

	if(GetFileTime(
		h_File,
		NULL,
		NULL,
		&f_time)==TRUE){
		if(FileTimeToSystemTime(
			&f_time,time)==TRUE)
			return true;
		else
			return false;
	}else{
		return false;
	}
}

/*
typedef enum {
	unknown = 0,
	html = 1,
	pdf = 2,
	gif = 3
} cont_types_t;
*/


void
respond::what_content_type(const request& rqu)
{
	std::string str;

	str=requested_file;

	_strlwr(const_cast<char*>(str.c_str()));

	str.erase(0, str.rfind('.')+1);

	content_type=unknown;

	if(str.size()>0)
	for(unsigned i=0; 
	i<sizeof(FileExtensions)/sizeof(file_ext_to_cont_type_t);
	i++){
		if(str.compare(FileExtensions[i].ext) == 0){
			content_type = FileExtensions[i].type;
		}
	}
}

void 
respond::content_type_to_str(const cont_types_t type, std::string* str)
{
	*str=ContentTypes[type];
}

bool 
respond::find_file(std::string name, bool* b_is_directory)
{
	WIN32_FIND_DATA data;
	HANDLE find_handle;

	find_handle=FindFirstFile(
		name.c_str(),
		&data);
	
	if(find_handle==INVALID_HANDLE_VALUE){
		DWORD err = GetLastError();
		return false;
	}
	
	*b_is_directory=false;
	if((data.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY){
		*b_is_directory=true;
	}
	
	FindClose(find_handle);
	
	return true;
}

bool 
respond::open_file(const request& rqu, char* szHttpErr, bool request_is_dir)
{
	b_have_body=true;
	string file_name=requested_path+requested_file;
	if(request_is_dir == true){
		file_name+='/';
		file_name+=index_file_name;
		content_type=html;
	}
	hFile=CreateFile(
		file_name.c_str(),
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,
		NULL);

	if(hFile == INVALID_HANDLE_VALUE){
		DWORD err =GetLastError();
	}

				
	if(hFile != INVALID_HANDLE_VALUE){

		bool bed_range=false;
		get_file_time(hFile, &file_time);

		if(rqu.type == request::get_if_modified){
			if(http_date(file_time) == rqu.remote_file_date){
				CloseHandle(hFile);
				hFile=INVALID_HANDLE_VALUE;
				return not_modified_stuff(rqu, szHttpErr);
			}
		}

		//
		
		file_size=GetFileSize(hFile, NULL);

		if(rqu.beg_pos>0){
			
			start_from=rqu.beg_pos;
			SetFilePointer(
				hFile,
				rqu.beg_pos,
				NULL,
				FILE_BEGIN);
		}else
			start_from=0;
		
		file_pos=start_from;
		
		file_end_pos=rqu.end_pos;
		
		if(rqu.instance_length>0){
			file_end_pos=rqu.instance_length-rqu.beg_pos;
		}
		
		if(file_end_pos>0){
			if(file_end_pos>file_size){
				bed_range=true;
			}
		}else
			file_end_pos=file_size;
		
		if(start_from>file_size ||
			rqu.beg_pos+rqu.instance_length>file_size){
			bed_range=true;
		}


		if(file_end_pos-start_from<file_size){
			err_code=206;
			wsprintf(szHttpErr, " %s", sz206);
		}else{
			err_code=200;
			wsprintf(szHttpErr, " %s", sz200);
		}

		if(bed_range){
			CloseHandle(hFile);
			hFile=INVALID_HANDLE_VALUE;
			return wrong_range_stuff(rqu, szHttpErr, 
				start_from, 
				file_end_pos, 
				rqu.instance_length);
		}

		if(rqu.type==request::head){
			file_pos=file_end_pos;
		}
		
		return true;
	}else
		return false;
}

bool 
respond::not_found_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=404;
	wsprintf(szHttpErr, " %s", sz404);
					
	if(make_temp_file() == true){

		std::string header="404";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>The requested URL ";
		stuff_str+="<strong>";
		stuff_str+=rqu.requested_target;
		stuff_str+="</strong>";
		stuff_str+=szHTML_BR;
		stuff_str+="was not found on this server.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::ditectory_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=200;
	wsprintf(szHttpErr, " %s", sz200);

	if(make_temp_file() == true){

		WIN32_FIND_DATA data;
		HANDLE find_handle;
		char szTmp[512];
		SYSTEMTIME time;
		unsigned __int64 total_size=0;
		unsigned long files_count=0;
		unsigned long dir_count=0;
		bool is_dir;
		bool is_root_dir(false);
		
		std::string dir_name=requested_path+requested_file;
		if(dir_name.compare(root_dir)==0){
			is_root_dir=true;
		}
		
		dir_name+="/";
		std::string path_prefix=dir_name;
		path_prefix.erase(0,root_dir.size());
		path_prefix=to_http_string(path_prefix);
		dir_name+="*.*";

		if(requested_file.size()>0){
			path_prefix+=requested_file;
			path_prefix+="/";
		}
		std::string link;
		

		find_handle=FindFirstFile(
			dir_name.c_str(),
			&data);
	
		if(find_handle!=INVALID_HANDLE_VALUE){
			DWORD err = GetLastError();
			
			{
				std::string head_str=host_name;
				head_str+=from_http_string(path_prefix);
				
				add_header_to_stuff(
					head_str,
					head_str);
			}
			
			do{
				if((data.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY){
					if(strlen(data.cFileName)==1 && data.cFileName[0]=='.')
						continue;
					if(is_root_dir && strcmp(data.cFileName, "..")==0)
						continue;
					dir_count++;
					is_dir=true;


					
				}else{
					files_count++;
					is_dir=false;
				}
				
				FileTimeToSystemTime(&data.ftLastWriteTime,&time);

				wsprintf(szTmp, 
					"%02u/%02u/%04u %02u:%02u:%02u ",
					time.wMonth,
					time.wDay,
					time.wYear,
					time.wHour,
					time.wMinute,
					time.wSecond);

				stuff_str+=szTmp;

				if(is_dir){
					wsprintf(szTmp, 
						"%10s ",
						" Folder");
				}else{
					wsprintf(szTmp, 
						"%9uk ",
						((data.nFileSizeLow&(~0x3ff))+0x400)/0x400);
					total_size+=data.nFileSizeLow;
				}

				stuff_str+=szTmp;
				stuff_str+="<a href=\"";
				stuff_str+=path_prefix;
				
				link=data.cFileName;
				link=to_http_string(link);

				stuff_str+=link;
				if(is_dir)stuff_str+='/';
				stuff_str+="\">";
				if(is_dir)	stuff_str+="<b>";
				stuff_str+=data.cFileName;
				if(is_dir)	stuff_str+="</b>";
				stuff_str+="</a>";
				stuff_str+=szHTML_BR;

			}while(FindNextFile(find_handle, &data) != 0);

			stuff_str+="<b>\r\n";
			stuff_str+=szHTML_BR;
			stuff_str+=szHTML_BR;
			sprintf(szTmp, 
				"File count: %u\r\nTotal size: %uk",
				files_count,
				((total_size&(~0x3ff))+0x400)/0x400);

			stuff_str+=szTmp;
			stuff_str+="</b>";

	
			FindClose(find_handle);

			close_stuff();
			
			write_stuff();
			return true;
		}
	}

	return false;
}

bool 
respond::make_temp_file()
{
	char szTempPath[1024];
	
	if(GetTempPath(sizeof(szTempPath), szTempPath)){
		char szTempFile[2048];
		
		if(GetTempFileName(
			szTempPath,
			"ruhttpsrv",
			0, szTempFile)){

			if(hFile != INVALID_HANDLE_VALUE)
				CloseHandle(hFile);
			
			hFile=CreateFile(
				szTempFile,
				GENERIC_READ|GENERIC_WRITE,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL|
				FILE_FLAG_SEQUENTIAL_SCAN|
				FILE_ATTRIBUTE_TEMPORARY|
				FILE_FLAG_DELETE_ON_CLOSE,
				NULL);

			if(hFile != INVALID_HANDLE_VALUE)
				return true;
		}
	}

	return false;
}

bool 
respond::write_stuff()
{
	if(hFile != INVALID_HANDLE_VALUE){
				
		get_file_time(hFile, &file_time);
				
		DWORD writen;
				
		if(WriteFile(hFile,
			stuff_str.c_str(),
			stuff_str.size(),
			&writen,
			NULL)){
			SetEndOfFile(hFile);
			file_size=GetFileSize(hFile, NULL);
			file_pos=0;
			file_end_pos=file_size;
			
			SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
			stuff_str.erase();
			content_type=html;
			return true;
		}
	}

	return false;
}

bool 
respond::parse_url(std::string& requested_target, bool* is_dir)
{
	string strTmp;
	int last_slash_pos=-1;
	int len=0;
	int target_size=requested_target.size();
	char c;

	if(target_size<=0){
		requested_file=root_dir;
		return false;
	}

	for(int it=0;
	it<target_size;
	it++){

		c=0;

		if( requested_target[it]=='%' ){
			char szHex[3];
			char* hexEndP;
			if(it<=target_size-3){
				it++;
				szHex[0]=requested_target[it++];
				szHex[1]=requested_target[it];
				szHex[2]=0;

				unsigned long cr=strtoul(szHex, &hexEndP, 16);
				if(cr<256){
					c=cr;
				}
			}
		}else{
			c=requested_target[it];
		}

		if(c=='/')last_slash_pos=len;
		if(c!=0){
			strTmp+=c;
			len++;
		}
	}

	requested_target=strTmp;

	requested_file.erase();
	requested_path=root_dir;
	if(last_slash_pos==-1)
		requested_path+='/';

	requested_path+=strTmp;

	char szTmpPath[2048];
	char* pszTmpPath=szTmpPath;
	char* pFile;

	len=requested_path.size();
	if(requested_path[len-1]=='/')
		*is_dir=true;

	DWORD FileNamePos = GetFullPathName(
		requested_path.c_str(),
		sizeof(szTmpPath),
		pszTmpPath,
		&pFile);

	if(FileNamePos>sizeof(szTmpPath)){
		pszTmpPath=(char*)alloca(FileNamePos+1);

		FileNamePos = GetFullPathName(
			requested_path.c_str(),
			FileNamePos,
			pszTmpPath,
			&pFile);
	}
	
	if(FileNamePos>0){
		len=strlen(pszTmpPath);
		if(pszTmpPath[len-1]=='\\'){len--;pszTmpPath[len]=0;}
		for(int i=0; i<len; i++){
			if(pszTmpPath[i]=='\\')
				pszTmpPath[i]='/';
		}
		requested_path=pszTmpPath;
		if(*is_dir==true){
			requested_path+='/';
			len++;
		}
		last_slash_pos=requested_path.rfind('/');
		if(last_slash_pos!=string::npos){
			last_slash_pos++;
			requested_file.append(requested_path, last_slash_pos, len);
			requested_path.erase(last_slash_pos);
		}
	}else{
		requested_path.erase();
	}

	if(root_dir.find_first_not_of(requested_path)!=string::npos){
		requested_path=root_dir;
		requested_path+='/';
		requested_file.erase();
		*is_dir=false;
		return false;
	}

	len=requested_path.size();
	if(*is_dir && len > 0 && requested_path[len-1]=='/')
		requested_path.erase(len-1);

	return true;
}

bool 
respond::not_modified_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=false;
	err_code=304;
	wsprintf(szHttpErr, " %s", sz304);

	return true;
}

bool 
respond::forbidden_stuff(const request& rqu, char* szHttpErr, bool reqwest_is_dir)
{
	b_have_body=true;
	err_code=403;
	wsprintf(szHttpErr, " %s", sz403);
					
	if(make_temp_file() == true){

		std::string header="403";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		if(reqwest_is_dir){
		}else{
		}
		stuff_str+="<p>Acces to requested URL ";
		stuff_str+="<strong>";
		stuff_str+=rqu.requested_target;
		stuff_str+="</strong>";
		stuff_str+=szHTML_BR;
		stuff_str+="forbidden.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::uri_too_long(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=414;
	wsprintf(szHttpErr, " %s", sz414);
					
	if(make_temp_file() == true){

		std::string header="414";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>Request URI: ";
		stuff_str+="<strong>";
		stuff_str+=rqu.requested_target;
		stuff_str+="</strong>";
		stuff_str+=szHTML_BR;
		stuff_str+="too long.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::redirect_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=302;
	wsprintf(szHttpErr, " %s", sz302);
	std::string new_location;

	new_location=host_name;
	new_location+=rqu.requested_target;
	new_location+='/';
	add_to_header="Location: ";
	add_to_header+=new_location;
	add_to_header+=szHttp_endl;

	if(make_temp_file() == true){

		std::string header="302";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>This document may be found here:";
		stuff_str+="<strong>";
		stuff_str+="<a href=\"";
		stuff_str+=new_location;
		stuff_str+="\">";
		stuff_str+=new_location;
		stuff_str+="</a>";
		stuff_str+="</strong>";
		stuff_str+=szHTML_BR;

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

void 
respond::clear()
{
	if(hFile!=INVALID_HANDLE_VALUE){
		CloseHandle(hFile);
		hFile=INVALID_HANDLE_VALUE;
	}

	err_code=0;
	file_size=0;
	file_pos=0;
	file_end_pos=0;
	start_from=0;
	cb_buffered=0;
	buff_pos=0;
	content_type=unknown;
}

bool 
respond::not_implemented_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=501;
	wsprintf(szHttpErr, " %s", sz501);
					
	if(make_temp_file() == true){

		std::string header="501";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>Requested function is not implemented on this server.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::not_metod_allowed_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=405;
	wsprintf(szHttpErr, " %s", sz405);
					
	if(make_temp_file() == true){

		std::string header="405";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>Requested metod is not allowed on this server.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::request_timeout_stuff(const request& rqu, char* szHttpErr)
{
	b_have_body=true;
	err_code=408;
	wsprintf(szHttpErr, " %s", sz408);
					
	if(make_temp_file() == true){

		std::string header="408";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>Request timeout.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}

bool 
respond::wrong_range_stuff(const request& rqu, char* szHttpErr, unsigned long start, unsigned long stop, unsigned long size)
{
	b_have_body=true;
	err_code=416;
	wsprintf(szHttpErr, " %s", sz416);
	char szRange[128];

	wsprintf(szRange, "%u-%u/%u", start, stop, size);
					
	if(make_temp_file() == true){

		std::string header="416";
		header+=szHttpErr;
		
		add_header_to_stuff(
			header,
			header);

		stuff_str+="<p>Requested range: ";
		stuff_str+="<strong>";
		stuff_str+=szRange;
		stuff_str+="</strong>";
		stuff_str+="not satisfiable.</p>";

		close_stuff();
		
		write_stuff();

		return true;
	}

	return false;
}