/*
**		Copyright (C) Roman Ukhov 2000
**		e-mail: r_ukhov@yahoo.com
**		http://rukhov.newmail.ru
*/

#if !defined(_AUT_C_SECTION_H_INCLUDED_)
#define _AUT_C_SECTION_H_INCLUDED_

namespace process
{

	class sinchronization_object
	{
		public:
			sinchronization_object(){}
			virtual ~sinchronization_object(){}

			virtual void lock() const = 0;
			virtual void unlock() const = 0;
		private:
			sinchronization_object(const sinchronization_object&){}
			const sinchronization_object& operator = (const sinchronization_object&){}
	};

	class critical_section : public sinchronization_object
	{
	public:
		critical_section(){
			InitializeCriticalSection(&_cs);
		}
		virtual ~critical_section(){
			DeleteCriticalSection(&_cs);
		}
		
		virtual void lock() const {
			EnterCriticalSection(&_cs);
		}
		virtual void unlock() const {
			LeaveCriticalSection(&_cs);
		}
	private:
		critical_section(const critical_section& x){}
		const critical_section& operator = (const critical_section& x) const {}
		mutable CRITICAL_SECTION _cs;
	};

	class auto_lock
	{
	public:
		auto_lock(const critical_section& cs) : _cs(cs){
			_cs.lock();
		}
		virtual ~auto_lock(){
			_cs.unlock();
		}

	private:
		const critical_section& _cs;
	};
}

#endif
